@@ -1,34 +1,79 @@
 """ test the scalar Timestamp """
 
 import calendar
-from datetime import datetime, timedelta
+from datetime import (
+    datetime,
+    timedelta,
+)
 import locale
+import pickle
 import unicodedata
 
 from dateutil.tz import tzutc
 import numpy as np
 import pytest
 import pytz
-from pytz import timezone, utc
+from pytz import (
+    timezone,
+    utc,
+)
 
-from pandas._libs.tslibs.timezones import dateutil_gettz as gettz, get_timezone
-from pandas.compat.numpy import np_datetime64_compat
+from pandas._libs.tslibs.timezones import (
+    dateutil_gettz as gettz,
+    get_timezone,
+)
+from pandas.compat import np_datetime64_compat
 import pandas.util._test_decorators as td
 
-from pandas import NaT, Timedelta, Timestamp
+from pandas import (
+    NaT,
+    Timedelta,
+    Timestamp,
+)
 import pandas._testing as tm
 
 from pandas.tseries import offsets
 
 
 class TestTimestampProperties:
+    def test_freq_deprecation(self):
+        # GH#41586
+        msg = "The 'freq' argument in Timestamp is deprecated"
+        with tm.assert_produces_warning(FutureWarning, match=msg):
+            # warning issued at construction
+            ts = Timestamp("2021-06-01", freq="D")
+            ts2 = Timestamp("2021-06-01", freq="B")
+
+        msg = "Timestamp.freq is deprecated"
+        with tm.assert_produces_warning(FutureWarning, match=msg):
+            # warning issued at attribute lookup
+            ts.freq
+
+        for per in ["month", "quarter", "year"]:
+            for side in ["start", "end"]:
+                attr = f"is_{per}_{side}"
+
+                with tm.assert_produces_warning(FutureWarning, match=msg):
+                    getattr(ts2, attr)
+
+                # is_(month|quarter|year)_(start|end) does _not_ issue a warning
+                #  with freq="D" bc the result will be unaffected by the deprecation
+                with tm.assert_produces_warning(None):
+                    getattr(ts, attr)
+
+    @pytest.mark.filterwarnings("ignore:The 'freq' argument:FutureWarning")
+    @pytest.mark.filterwarnings("ignore:Timestamp.freq is deprecated:FutureWarning")
     def test_properties_business(self):
         ts = Timestamp("2017-10-01", freq="B")
         control = Timestamp("2017-10-01")
         assert ts.dayofweek == 6
         assert ts.day_of_week == 6
         assert not ts.is_month_start  # not a weekday
+        assert not ts.freq.is_month_start(ts)
+        assert ts.freq.is_month_start(ts + Timedelta(days=1))
         assert not ts.is_quarter_start  # not a weekday
+        assert not ts.freq.is_quarter_start(ts)
+        assert ts.freq.is_quarter_start(ts + Timedelta(days=1))
         # Control case: non-business is month/qtr start
         assert control.is_month_start
         assert control.is_quarter_start
@@ -38,7 +83,11 @@ class TestTimestampProperties:
         assert ts.dayofweek == 5
         assert ts.day_of_week == 5
         assert not ts.is_month_end  # not a weekday
+        assert not ts.freq.is_month_end(ts)
+        assert ts.freq.is_month_end(ts - Timedelta(days=1))
         assert not ts.is_quarter_end  # not a weekday
+        assert not ts.freq.is_quarter_end(ts)
+        assert ts.freq.is_quarter_end(ts - Timedelta(days=1))
         # Control case: non-business is month/qtr start
         assert control.is_month_end
         assert control.is_quarter_end
@@ -385,10 +434,23 @@ class TestTimestamp:
 
     def test_tz_conversion_freq(self, tz_naive_fixture):
         # GH25241
-        t1 = Timestamp("2019-01-01 10:00", freq="H")
-        assert t1.tz_localize(tz=tz_naive_fixture).freq == t1.freq
-        t2 = Timestamp("2019-01-02 12:00", tz="UTC", freq="T")
-        assert t2.tz_convert(tz="UTC").freq == t2.freq
+        with tm.assert_produces_warning(FutureWarning, match="freq"):
+            t1 = Timestamp("2019-01-01 10:00", freq="H")
+            assert t1.tz_localize(tz=tz_naive_fixture).freq == t1.freq
+        with tm.assert_produces_warning(FutureWarning, match="freq"):
+            t2 = Timestamp("2019-01-02 12:00", tz="UTC", freq="T")
+            assert t2.tz_convert(tz="UTC").freq == t2.freq
+
+    def test_pickle_freq_no_warning(self):
+        # GH#41949 we don't want a warning on unpickling
+        with tm.assert_produces_warning(FutureWarning, match="freq"):
+            ts = Timestamp("2019-01-01 10:00", freq="H")
+
+        out = pickle.dumps(ts)
+        with tm.assert_produces_warning(None):
+            res = pickle.loads(out)
+
+        assert res._freq == ts._freq
 
 
 class TestTimestampNsOperations:
@@ -494,32 +556,32 @@ class TestTimestampConversion:
         ts = Timestamp("2011-01-01 9:00:00.123456789")
 
         # Warn the user of data loss (nanoseconds).
-        with tm.assert_produces_warning(UserWarning, check_stacklevel=False):
+        with tm.assert_produces_warning(UserWarning):
             expected = datetime(2011, 1, 1, 9, 0, 0, 123456)
             result = ts.to_pydatetime()
             assert result == expected
 
     def test_timestamp_to_datetime(self):
-        stamp = Timestamp("20090415", tz="US/Eastern", freq="D")
+        stamp = Timestamp("20090415", tz="US/Eastern")
         dtval = stamp.to_pydatetime()
         assert stamp == dtval
         assert stamp.tzinfo == dtval.tzinfo
 
     def test_timestamp_to_datetime_dateutil(self):
-        stamp = Timestamp("20090415", tz="dateutil/US/Eastern", freq="D")
+        stamp = Timestamp("20090415", tz="dateutil/US/Eastern")
         dtval = stamp.to_pydatetime()
         assert stamp == dtval
         assert stamp.tzinfo == dtval.tzinfo
 
     def test_timestamp_to_datetime_explicit_pytz(self):
-        stamp = Timestamp("20090415", tz=pytz.timezone("US/Eastern"), freq="D")
+        stamp = Timestamp("20090415", tz=pytz.timezone("US/Eastern"))
         dtval = stamp.to_pydatetime()
         assert stamp == dtval
         assert stamp.tzinfo == dtval.tzinfo
 
     @td.skip_if_windows_python_3
     def test_timestamp_to_datetime_explicit_dateutil(self):
-        stamp = Timestamp("20090415", tz=gettz("US/Eastern"), freq="D")
+        stamp = Timestamp("20090415", tz=gettz("US/Eastern"))
         dtval = stamp.to_pydatetime()
         assert stamp == dtval
         assert stamp.tzinfo == dtval.tzinfo
@@ -528,18 +590,21 @@ class TestTimestampConversion:
         # Ensure that converting to datetime and back only loses precision
         # by going from nanoseconds to microseconds.
         exp_warning = None if Timestamp.max.nanosecond == 0 else UserWarning
-        with tm.assert_produces_warning(exp_warning, check_stacklevel=False):
-            assert (
-                Timestamp(Timestamp.max.to_pydatetime()).value / 1000
-                == Timestamp.max.value / 1000
-            )
+        with tm.assert_produces_warning(exp_warning):
+            pydt_max = Timestamp.max.to_pydatetime()
+
+        assert Timestamp(pydt_max).value / 1000 == Timestamp.max.value / 1000
 
         exp_warning = None if Timestamp.min.nanosecond == 0 else UserWarning
-        with tm.assert_produces_warning(exp_warning, check_stacklevel=False):
-            assert (
-                Timestamp(Timestamp.min.to_pydatetime()).value / 1000
-                == Timestamp.min.value / 1000
-            )
+        with tm.assert_produces_warning(exp_warning):
+            pydt_min = Timestamp.min.to_pydatetime()
+
+        # The next assertion can be enabled once GH#39221 is merged
+        #  assert pydt_min < Timestamp.min  # this is bc nanos are dropped
+        tdus = timedelta(microseconds=1)
+        assert pydt_min + tdus > Timestamp.min
+
+        assert Timestamp(pydt_min + tdus).value / 1000 == Timestamp.min.value / 1000
 
     def test_to_period_tz_warning(self):
         # GH#21333 make sure a warning is issued when timezone
